#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/stat.h>

#define MAP_SIZE 100

int main(int argc, char *argv[])
{
    if (argc < 2)
    {
        printf("*** Usage: filepath new_contents\n");
        return -1;
    }
    
    char *filepath     = argv[1];
    char *new_contents = argv[2];

    int fd = open(filepath, O_RDWR);
    if (fd == -1)
    {
        printf("*** open() failed: %s\n", filepath);
        return -2;
    }
    
    // Map the file into memory.
    char *addr = mmap(NULL, MAP_SIZE, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0);
    if (addr == MAP_FAILED)
    {
        printf("*** mmap() failed.\n");
        return -3;
    }
    
    // File descriptor no longer needed.
    if (close(fd) == -1)
    {
        printf("*** close(fd) failed.\n");
        return -4;
    }
    
    printf("Current contents of file %s: \"%.*s\"\n", filepath, MAP_SIZE, addr);

    // Zero out the mapped memory and copy in the new contents.
    memset(addr, 0, MAP_SIZE);
    strncpy(addr, argv[2], MAP_SIZE-1);

    printf("Copied \"%s\" to mapped file.\n", new_contents);
    
    // Flush back out to the disk file.
    if (msync(addr, MAP_SIZE, MS_SYNC) == -1)
    {
        printf("*** msync() failed.\n");
        return -5;
    }
    
    return 0;
}
